﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using TCatSysManagerLib;
using Beckhoff.Forms;
using ECTOPOLOGYLib;

namespace Messages
{
	public partial class FormMessagesAndDiagnosis : Form
	{
		public static Settings appSettings;
		public static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
		bool mustSaveSettings = false;

		string eventLogStatusTag;

		private static TcSysManager sysManager;
		ITcSysManager2 sysManager2;
		ITcSmTreeItem TIID;
		int OfflineDeviceId;
		int OnlineDeviceId;
	
		TcAdsPlcServer adsPlcServer;
		public FormMessagesAndDiagnosis()
		{
			InitializeComponent();
			string amsNetId = appSettings.amsNetId;
			int adsPortNr;
			if (int.TryParse(appSettings.adsPortNr, out adsPortNr) == false)
				adsPortNr = 801;
			if (!string.IsNullOrEmpty(amsNetId))
				tcEventLoggerData1.Connection = Beckhoff.EventLogger.ConnectionType.Ads;
			tcEventLoggerData1.AdsServerNetId = amsNetId;
			tcEventLoggerData1.UseDocLinks = appSettings.eventlogUseDocLinks;
			tcEventLoggerData1.EventChanged += new Beckhoff.EventLogger.TcEventChangedEventHandler(tcEventLoggerData1_EventChanged);
			tcEventLoggerData1.InstructionEventChanged += new Beckhoff.EventLogger.TcEventChangedEventHandler(tcEventLoggerData1_InstructionEventChanged);

			eventLogStatusTag = this.Name + ".labelEventLogStatus.ActiveEvents";
			labelEventLogStatus.Text = eventLogStatusTag;

			InitLcid();
			SetLoggedFKeyButtonState(false);
			tcLog4NetView1.FileName = GetLogFile();

			tcEventLoggerList1.ColumnReordered += new ColumnReorderedEventHandler(tcEventLoggerList1_ColumnReordered);
			tcEventLoggerList1.ColumnWidthChanging += new ColumnWidthChangingEventHandler(tcEventLoggerList1_ColumnWidthChanging);

			sysManager = new TcSysManager();
			sysManager2 = (ITcSysManager2)sysManager;

			tbNetID.Text = amsNetId;
			adsPlcServer = new TcAdsPlcServer(adsPortNr, amsNetId);
			adsPlcServer.TcPlcStateChanged += new Beckhoff.Forms.TcAdsStateChangedEventHandler(adsPlcServer_TcPlcStateChanged);
		}

		private void adsPlcServer_TcPlcStateChanged(TwinCAT.Ads.AdsState state)
		{
			if (state == TwinCAT.Ads.AdsState.Run)
			{
				lbStatus.BackColor = Color.Green;
			}
			else
            {
				lbStatus.BackColor = Color.Red;
			}
		}
				void InitLcid()
		{
			// lcid comes from settings (else language manager)
			if (appSettings.eventlogUseTclmLCID == false)
			{
				int lcid = 1033;
				comboBoxLanguage.SelectedIndex = 1;
				tcEventLoggerData1.LanguageId = lcid;
				tcEventLoggerData1.GetActiveEvents();
			}
		}
		void tcEventLoggerList1_ColumnWidthChanging(object sender, ColumnWidthChangingEventArgs e)
		{
			mustSaveSettings = true;
		}

		void tcEventLoggerList1_ColumnReordered(object sender, ColumnReorderedEventArgs e)
		{
			mustSaveSettings = true;
		}
		string GetLogFile()
		{
			log4net.Appender.IAppender[] appenders = log.Logger.Repository.GetAppenders();
			foreach (log4net.Appender.IAppender appender in appenders)
			{
				if (appender is log4net.Appender.RollingFileAppender)
				{
					log4net.Appender.RollingFileAppender rfa = (log4net.Appender.RollingFileAppender)appender;
					return rfa.File;
				}
			}
			return "";
		}

        private void FormMessage_Load(object sender, EventArgs e)
        {
			tcEventLoggerList1.Focus();
			SetViewSettings();
		}


		public void ViewActive()
		{
			eventLogStatusTag = this.Name + ".labelEventLogStatus.ActiveEvents";
			labelEventLogStatus.Text = eventLogStatusTag;
			tcEventLoggerData1.GetActiveEvents();  // not absolutely necessary
			tcEventLoggerList1.Focus();
			tcLog4NetView1.ShowEvents(false);
			SetLoggedFKeyButtonState(false);
		}

		public void ViewLogged()
		{
			eventLogStatusTag = this.Name + ".labelEventLogStatus.LoggedEvents";
			labelEventLogStatus.Text = eventLogStatusTag;
			tcLog4NetView1.ShowEvents(true);
			SetLoggedFKeyButtonState(true);
		}

		public void ShowDetails()
		{
			if (tcLog4NetView1.Visible == true)
			{
				tcLog4NetView1.ShowDetailWindow = !tcLog4NetView1.ShowDetailWindow;
				tcLog4NetView1.Focus();
			}
			else
			{
				tcEventLoggerList1.ShowDetailWindow = !tcEventLoggerList1.ShowDetailWindow;
				tcEventLoggerList1.Focus();
			}
		}
		public void ClearLogged()
		{
			try
			{
				string logFile = tcLog4NetView1.FileName;
				DialogResult msgBoxRes = new DialogResult();
				// make a copy
				string tmp1 = "Do you want to save the logfile before clearing it?";
				string tmp2 = "Save Event Log File";
				msgBoxRes = MessageBox.Show(tmp1, tmp2, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
				if (msgBoxRes == DialogResult.Yes)
				{
					SaveFileDialog dlg = new SaveFileDialog();
					dlg.Filter = "LOG files (*.log)|*.log|All files (*.*)|*.*"; ;
					dlg.RestoreDirectory = true;
					if (dlg.ShowDialog() == DialogResult.OK)
					{
						System.IO.File.Copy(logFile, dlg.FileName, true);
					}
				}

				// delete logfile
				if (msgBoxRes == DialogResult.Yes
					|| msgBoxRes == DialogResult.No)
				{
					log4net.LogManager.GetRepository().ResetConfiguration();
					log4net.LogManager.GetRepository().Shutdown();
					System.IO.File.Delete(logFile);

					log4net.Config.XmlConfigurator.Configure(new System.IO.FileInfo(Application.StartupPath + "\\System\\log.xml"));
					
					tcLog4NetView1.ShowEvents();
				}
			}
			catch { }
		}

		public void SaveLogged()
		{
			try
			{
				// make a copy
				string logFile = tcLog4NetView1.FileName;
				SaveFileDialog dlg = new SaveFileDialog();
				dlg.Filter = "LOG files (*.log)|*.log|All files (*.*)|*.*"; ;
				dlg.RestoreDirectory = true;
				if (dlg.ShowDialog() == DialogResult.OK)
				{
					System.IO.File.Copy(logFile, dlg.FileName, true);
				}
			}
			catch { }
		}


		private void SetLoggedFKeyButtonState(bool bEnable)
        {
			btClearLogged.Enabled = bEnable;
			btSaveLogged.Enabled = bEnable;
		}

        private void FormMessage_Activated(object sender, EventArgs e)
        {
			if (tcLog4NetView1.Visible == true)
				tcLog4NetView1.ShowEvents();
		}

        private void FormMessage_Deactivate(object sender, EventArgs e)
        {
			SaveSettings();
		}

		void SetViewSettings()
		{
			int nMin = appSettings.eventlogColWidth.GetLowerBound(0);
			int nMax = appSettings.eventlogColWidth.GetUpperBound(0);
			for (int i = nMin; i <= nMax; i++)
			{
				if (appSettings.eventlogColWidth[i] != -1)
					tcEventLoggerList1.SetListViewColumn(i, appSettings.eventlogColWidth[i]);
				tcEventLoggerList1.SetListViewDisplayIndex(i, appSettings.eventlogColDisplayOrder[i]);
			}
			tcEventLoggerList1.ShowCol = appSettings.eventlogColStatus;
		}

		void SaveSettings()
		{
			if (mustSaveSettings == false)
				return;

			mustSaveSettings = false;
			int nMin = appSettings.eventlogColWidth.GetLowerBound(0);
			int nMax = appSettings.eventlogColWidth.GetUpperBound(0);
			for (int i = nMin; i <= nMax; i++)
			{
				appSettings.eventlogColWidth[i] = tcEventLoggerList1.SetListViewColumn(i, -1);
				appSettings.eventlogColDisplayOrder[i] = tcEventLoggerList1.SetListViewDisplayIndex(i, -1);
			}
		}

		private void tcEventLoggerData1_EventChanged(Beckhoff.EventLogger.TcEventItem eventItem, Beckhoff.EventLogger.TcEventType eventType)
		{
			if (eventItem == null)
				return;

			// route message to the log4net, if an error happens...
			if (eventItem.Class == (int)Beckhoff.EventLogger.TcEventClass.Alarm && !string.IsNullOrEmpty(eventItem.Message))
				log.Error(eventItem.Message);

			
		}

		private void tcEventLoggerData1_InstructionEventChanged(Beckhoff.EventLogger.TcEventItem eventItem, Beckhoff.EventLogger.TcEventType eventType)
		{
			if (eventItem == null)
				return;

		}

        private void btViewActive_Click(object sender, EventArgs e)
        {
			ViewActive();
        }

        private void btViewLogged_Click(object sender, EventArgs e)
        {
			ViewLogged();
        }

        private void btShowDetails_Click(object sender, EventArgs e)
        {
			ShowDetails();
        }

        private void btClearLogged_Click(object sender, EventArgs e)
        {
			ClearLogged();
        }

        private void btSaveLogged_Click(object sender, EventArgs e)
        {
			SaveLogged();
        }

        private void comboBoxLanguage_SelectedIndexChanged(object sender, EventArgs e)
        {
			int lcid = 1033;  // default English
			
			if (comboBoxLanguage.SelectedIndex != -1)
				int.TryParse((string)comboBoxLanguage.SelectedItem, out lcid);

			if (tcEventLoggerData1.LanguageId != lcid)
			{
				tcEventLoggerData1.LanguageId = lcid;
				tcEventLoggerData1.GetActiveEvents();
			}
		}

		private void btShowOffline_Click(object sender, EventArgs e)
		{

			try
			{

				tbOnlineDeviceID.Text = "";
				cbShowOffline.Items.Clear();
				axEcTopologyCtrl1.ZoomFactor = 3;

				axEcTopologyCtrl1.ShowOfflineTopology(null);//清空显示
															//获取离线的所有设备
				TIID = sysManager.LookupTreeItem("TIID");
				for (int i = 1; i <= TIID.ChildCount; i++)
				{
					ITcSmTreeItem chlid = TIID.Child[i];
					cbShowOffline.Items.Add(i);
				}
				cbShowOffline.DroppedDown = true;
				btShowOnline.BackColor = SystemColors.GradientActiveCaption;
				btShowOffline.BackColor = Color.Blue;
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void cbShowOffline_SelectedIndexChanged(object sender, EventArgs e)
		{
			try
			{
				OfflineDeviceId = int.Parse(cbShowOffline.Text);
				tbOfflineDeviceID.Text = OfflineDeviceId.ToString();
				axEcTopologyCtrl1.ShowOfflineTopology(TIID.Child[OfflineDeviceId]);
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void btShowOnline_Click(object sender, EventArgs e)
		{
			try
			{
				tbOfflineDeviceID.Text = "";
				axEcTopologyCtrl1.TargetNetId = adsPlcServer.ClientNetId;
				cbShowOnline.Items.Clear();
				axEcTopologyCtrl1.ShowOfflineTopology(null);//清空显示
															//axEcTopologyCtrl1.ShowCompareView();拓扑对比

				//获取在线的所有设备
				IEnumEtherCATDevices devices;
				IEtherCATDevice device;
				axEcTopologyCtrl1.GetEnumEtherCATDevices(out devices);
				uint fetched = 0;
				do
				{
					devices.Next(1, out device, out fetched);
					if (fetched == 1)
						cbShowOnline.Items.Insert(0, device.DeviceId);
				} while (fetched == 1);

				axEcTopologyCtrl1.ShowSlaveInfo = true;//显示从站信息对话框
				axEcTopologyCtrl1.ShowCrcErrorMarker = true;//显示CrcError错误标记                
				axEcTopologyCtrl1.ShowAdvancedSettingsMenu = true;//测试未发现具体显示内容
																  //定义了是否应将条目 AdvancedSettings...添加到 EtherCAT 从站的上下文菜单中。
																  //如果选择此菜单项，则会触发 OnAdvancedSettings 事件

				cbShowOnline.DroppedDown = true;
				btShowOnline.BackColor = Color.Blue;
				btShowOffline.BackColor = SystemColors.GradientActiveCaption;
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void cbShowOnline_SelectedIndexChanged(object sender, EventArgs e)
		{
			try
			{
				OnlineDeviceId = int.Parse(cbShowOnline.Text);
				tbOnlineDeviceID.Text = OnlineDeviceId.ToString();
				axEcTopologyCtrl1.DeviceId = int.Parse(cbShowOnline.Text);
				axEcTopologyCtrl1.ShowOnlineTopology();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

        private void btOpenTSM_Click(object sender, EventArgs e)
        {
			if (openFileDialog1.ShowDialog() == DialogResult.OK)
			{
				tbPath.Text = openFileDialog1.FileName;
				sysManager2.OpenConfiguration(openFileDialog1.FileName);
			}
		}

       
    }
}
