﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Messages
{
    public class Settings
    {
        private string fileName;
		private Beckhoff.App.Setting.Xml AppSett;
		private int MaxEventLogListColumns;
		public bool[] eventlogColStatus;
		public int[] eventlogColWidth;
		public int[] eventlogColDisplayOrder;
		public int eventlogLanguage;
		public bool eventlogUseTclmLCID;
		public string[] eventlogLanguages;
		public int eventlogRefreshTime;
		public bool eventlogUseDocLinks;

		public string amsNetId;
		/// <summary>
		/// The Ads port nr.
		/// </summary>
		public string adsPortNr;

		public Settings()
		{
			MaxEventLogListColumns = Beckhoff.EventLogger.TcEventLogList.MaxColumns;
			
			eventlogColStatus = new bool[MaxEventLogListColumns];
			eventlogColWidth = new int[MaxEventLogListColumns];
			eventlogColDisplayOrder = new int[MaxEventLogListColumns];
			
			eventlogLanguage = 0;
			
		}

		public string FileName
		{
			get
			{
				return fileName;
			}
			set
			{
				fileName = value;
				// the setting instance
				AppSett = null;
				AppSett = new Beckhoff.App.Setting.Xml(fileName);
				AppSett.Buffer();
			}
		}

		public bool WriteSettings()
		{
			if (AppSett == null)
				return false;

			
			WriteEventLogSettings();
			WriteGeneralSettings();

			// must be at the end
			AppSett.Buffer().Flush();
			return true;
		}

		

		public bool ReadSettings()
		{
			if (AppSett == null)
				return false;

			FormMessagesAndDiagnosis.log.Info("Settings loaded: " + fileName);

			// must be at first
			if (AppSett.Buffer().NeedsFlushing)
				AppSett.Buffer().Flush();

			
			ReadEventLogSettings();
			ReadGeneralSettings();

			return true;
		}

		void WriteEventLogSettings()
		{
			string tmp = "";
			string tmp2 = "";

			int nMin = eventlogColStatus.GetLowerBound(0);
			int nMax = eventlogColStatus.GetUpperBound(0);
			for (int i = nMin; i <= nMax; i++)
			{
				AppSett.SetValue("EventLogger", "Col" + i.ToString(), eventlogColStatus[i]);
				tmp += eventlogColWidth[i].ToString() + ";";
				tmp2 += eventlogColDisplayOrder[i].ToString() + ";";
			}

			AppSett.SetValue("EventLogger", "ColWidth", tmp);
			AppSett.SetValue("EventLogger", "DisplayOrder", tmp2);
			AppSett.SetValue("EventLogger", "RefreshTime", eventlogRefreshTime);
			AppSett.SetValue("EventLogger", "Language", eventlogLanguage);
			AppSett.SetValue("EventLogger", "UseTclmLCID", eventlogUseTclmLCID);
			string tmpLang = "";
			if (eventlogLanguages.GetLength(0) > 1)
			{
				foreach (string item in eventlogLanguages)
					tmpLang += item + ";";
				tmpLang = tmpLang.Remove(tmpLang.LastIndexOf(';'), 1);
				AppSett.SetValue("EventLogger", "Languages", tmpLang);
			}
			AppSett.SetValue("EventLogger", "UseDocLinks", eventlogUseDocLinks);
		}
		void ReadEventLogSettings()
		{
			int value = 0;
			string tmp = AppSett.GetValue("EventLogger", "ColWidth", "");
			string tmp2 = AppSett.GetValue("EventLogger", "DisplayOrder", "");
			string[] tmpX = tmp.Split(';');
			string[] tmp2X = tmp2.Split(';');

			int nMin = eventlogColStatus.GetLowerBound(0);
			int nMax = eventlogColStatus.GetUpperBound(0);
			for (int i = nMin; i <= nMax; i++)
			{
				eventlogColStatus[i] = AppSett.GetValue("EventLogger", "Col" + i.ToString(), true);
			}

			for (int i = 0; i < MaxEventLogListColumns; i++)
			{
				eventlogColDisplayOrder[i] = i;
				eventlogColWidth[i] = -1;
			}

			nMin = tmpX.GetLowerBound(0);
			nMax = tmpX.GetUpperBound(0);
			if (nMax > eventlogColWidth.GetUpperBound(0))
				nMax = eventlogColWidth.GetUpperBound(0);

			for (int i = nMin; i <= nMax; i++)
			{
				value = -1;
				if (int.TryParse(tmpX[i], out value))
					eventlogColWidth[i] = value;
			}

			nMin = tmp2X.GetLowerBound(0);
			nMax = tmp2X.GetUpperBound(0);
			if (nMax > eventlogColDisplayOrder.GetUpperBound(0))
				nMax = eventlogColDisplayOrder.GetUpperBound(0);

			for (int i = nMin; i <= nMax; i++)
			{
				value = i + 1;
				if (int.TryParse(tmp2X[i], out value))
					eventlogColDisplayOrder[i] = value;
			}

			eventlogRefreshTime = AppSett.GetValue("EventLogger", "RefreshTime", 1500);
			eventlogLanguage = AppSett.GetValue("EventLogger", "Language", 0);
			string tmpLanguages = AppSett.GetValue("EventLogger", "Languages", "1031;1033;1034;1036;1040;1053;2052;1049;1026");
			eventlogLanguages = tmpLanguages.Split(';');
			eventlogUseDocLinks = AppSett.GetValue("EventLogger", "UseDocLinks", false);
			eventlogUseTclmLCID = AppSett.GetValue("EventLogger", "UseTclmLCID", false);
		}

		void ReadGeneralSettings()
		{
			amsNetId = AppSett.GetValue("General", "AmsNetId", "");
			adsPortNr = AppSett.GetValue("General", "AdsPortNr", "801");

		}

		void WriteGeneralSettings()
		{
			AppSett.SetValue("General", "AmsNetId", amsNetId);
			AppSett.SetValue("General", "AdsPortNr", adsPortNr.Trim());

		}
	}
}
